@@ -142,16 +142,11 @@ extern GtkWidget *hw_spdif_output_notebook;
 
 extern GtkWidget *hw_spdif_input_coaxial_radio;
 extern GtkWidget *hw_spdif_input_optical_radio;
-
-extern GtkWidget *hw_breakbox_led_on_radio;
-extern GtkWidget *hw_breakbox_led_off_radio;
+extern GtkWidget *hw_spdif_switch_off_radio;
 
 extern GtkWidget *hw_phono_input_on_radio;
 extern GtkWidget *hw_phono_input_off_radio;
 
-extern GtkWidget *hw_spdif_switch_on_radio;
-extern GtkWidget *hw_spdif_switch_off_radio;
-
 extern GtkWidget *input_interface_internal;
 extern GtkWidget *input_interface_front_input;
 extern GtkWidget *input_interface_rear_input;
@@ -165,6 +160,9 @@ extern GtkLabel *av_ipga_volume_label[];
 extern GtkWidget *av_dac_sense_radio[][4];
 extern GtkWidget *av_adc_sense_radio[][4];
 
+/* flags */
+extern int card_is_dmx6fire;
+
 gint level_meters_configure_event(GtkWidget *widget, GdkEventConfigure *event);
 gint level_meters_expose_event(GtkWidget *widget, GdkEventExpose *event);
 gint level_meters_timeout_callback(gpointer data);
@@ -210,8 +208,6 @@ void spdif_output_toggled(GtkWidget *togglebutton, gpointer data);
 void spdif_input_update(void);
 void spdif_input_toggled(GtkWidget *togglebutton, gpointer data);
 void analog_input_select_toggled(GtkWidget *togglebutton, gpointer data);
-void breakbox_led_toggled(GtkWidget *togglebutton, gpointer data);
-void spdif_on_off_toggled(GtkWidget *togglebutton, gpointer data);
 void phono_input_toggled(GtkWidget *togglebutton, gpointer data);
 
 void hardware_init(void);
@@ -221,6 +217,7 @@ void analog_volume_postinit(void);
 int envy_dac_volumes(void);
 int envy_dac_max(void);
 int envy_adc_volumes(void);
+int envy_adc_max(void);
 int envy_ipga_volumes(void);
 int envy_dac_senses(void);
 int envy_adc_senses(void);
@@ -243,3 +240,4 @@ void adc_sense_toggled(GtkWidget *togglebutton, gpointer data);
 
 void control_input_callback(gpointer data, gint source, GdkInputCondition condition);
 void mixer_input_callback(gpointer data, gint source, GdkInputCondition condition);
+
